/* $Id: GSM.xs,v 1.2 2001/10/25 16:07:08 elwood Exp $ */

#undef VERSION
#ifdef __cplusplus
extern "C" {
#endif
#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"
#ifdef __cplusplus
}
#endif
#include <time.h>
#define false (0)
#define true (!false)
#undef _
#include <gsm-common.h>
#include <gsm-api.h>

typedef struct {
  int Empty;                                       /* Is this entry empty? */
  char Name[GSM_MAX_PHONEBOOK_NAME_LENGTH + 1];     /* Plus 1 for null terminator. */
  char Number[GSM_MAX_PHONEBOOK_NUMBER_LENGTH + 1]; /* Number */
  GSM_MemoryType MemoryType;                        /* Type of memory (SIM/internal) */
  int Group;                                        /* Group */
  int Location;
} Perl_GSM_PhonebookEntry;

typedef struct {
  int AlarmEnabled; /* Is the alarm set? */
  int Year;          /* The complete year specification - e.g. 1999. Y2K :-) */
  int Month;	     /* January = 1 */
  int Day;
  int Hour;
  int Minute;
  int Second;
  int Timezone;
} Perl_GSM_DateTime;

#define GSMDATE_TO_TM(GSMP,TM,HASH)\
{\
  time_t rt;\
\
  TM.tm_year = GSMP->Year+2000;\
  if(TM.tm_year > 1900)\
    TM.tm_year -= 1900;\
  TM.tm_mon  = GSMP->Month - 1;\
  TM.tm_mday = GSMP->Day;\
  TM.tm_hour = GSMP->Hour;\
  TM.tm_min  = GSMP->Minute;\
  TM.tm_sec  = GSMP->Second;\
  TM.tm_isdst=-1;\
  TM.tm_wday = 0;\
  TM.tm_yday = 0;\
  rt = mktime(&TM);\
  hv_store(HASH,"Date",4,newSViv(rt),0);\
}

#define HASH_TO_GSMDT(GSMDT,LOCALTIME)\
{\
  struct tm *t;\
  t = (struct tm *)malloc(sizeof(struct tm));\
  t = localtime(LOCALTIME);\
  GSMDT->Year = 1900+t->tm_year;\
  GSMDT->Month = t->tm_mon + 1;\
  GSMDT->Day = t->tm_mday;\
  GSMDT->Hour = t->tm_hour;\
  GSMDT->Minute = t->tm_min;\
  GSMDT->Second = t->tm_sec;\
}


typedef struct {
  Perl_GSM_DateTime Time;	            /* Date of reception/response of messages. */
  Perl_GSM_DateTime SMSCTime;
  int Length;                               /* Length of the SMS message. */
  int Validity;                             /* Validity Period of the SMS message. */
  GSM_UDH UDHType;                          /* If UDH is present - type of UDH */                    
  char UDH[GSM_MAX_USER_DATA_HEADER_LENGTH];
  char MessageText[GSM_MAX_SMS_LENGTH + 1]; /* Room for null term. */
  GSM_MessageCenter MessageCenter;          /* SMS Center. */
  char Sender[GSM_MAX_SENDER_LENGTH + 1];   /* Sender of the SMS message. */
  char Destination[GSM_MAX_DESTINATION_LENGTH + 1]; /* Destination of the message. */
  int MessageNumber;                        /* Location in the memory. */
  GSM_MemoryType MemoryType;                /* Type of memory message is stored in. */
  GSM_SMSMessageType Type;                  /* Type of the SMS message */
  GSM_SMSMessageStatus Status;              /* Status of the SMS message */
  int Class;                                /* Class Message: 0, 1, 2, 3 or none; see GSM 03.38 */
  int EightBit;                            /* Indicates whether SMS contains 8 bit data or not */
  int Compression;                         /* Indicates whether SMS contains compressed data or not */
  int Location;
  int ReplyViaSameSMSC;
} Perl_GSM_SMSMessage;


static int
not_here(s)
char *s;
{
    croak("%s not implemented on this architecture", s);
    return -1;
}
static double
constant(name, arg)
char *name;
int arg;
{
    errno = 0;
    switch (*name) {
    case 'A':
	break;
    case 'B':
	break;
    case 'C':
	break;
    case 'D':
	break;
    case 'E':
	break;
    case 'F':
	break;
    case 'G':
	if (strEQ(name, "GSM_GROUPS_NOT_SUPPORTED"))
#ifdef GSM_GROUPS_NOT_SUPPORTED
	    return GSM_GROUPS_NOT_SUPPORTED;
#else
	    goto not_there;
#endif
	if (strEQ(name, "GSM_MAX_DESTINATION_LENGHT"))
#ifdef GSM_MAX_DESTINATION_LENGHT
	    return GSM_MAX_DESTINATION_LENGHT;
#else
	    goto not_there;
#endif
	if (strEQ(name, "GSM_MAX_DEVICE_NAME_LENGTH"))
#ifdef GSM_MAX_DEVICE_NAME_LENGTH
	    return GSM_MAX_DEVICE_NAME_LENGTH;
#else
	    goto not_there;
#endif
	if (strEQ(name, "GSM_MAX_PHONEBOOK_NAME_LENGTH"))
#ifdef GSM_MAX_PHONEBOOK_NAME_LENGTH
	    return GSM_MAX_PHONEBOOK_NAME_LENGTH;
#else
	    goto not_there;
#endif
	if (strEQ(name, "GSM_MAX_PHONEBOOK_NUMBER_LENGTH"))
#ifdef GSM_MAX_PHONEBOOK_NUMBER_LENGTH
	    return GSM_MAX_PHONEBOOK_NUMBER_LENGTH;
#else
	    goto not_there;
#endif
	if (strEQ(name, "GSM_MAX_SENDER_LENGTH"))
#ifdef GSM_MAX_SENDER_LENGTH
	    return GSM_MAX_SENDER_LENGTH;
#else
	    goto not_there;
#endif
	if (strEQ(name, "GSM_MAX_SMS_CENTRE_LENGTH"))
#ifdef GSM_MAX_SMS_CENTRE_LENGTH
	    return GSM_MAX_SMS_CENTRE_LENGTH;
#else
	    goto not_there;
#endif
	if (strEQ(name, "GSM_MAX_SMS_LENGTH"))
#ifdef GSM_MAX_SMS_LENGTH
	    return GSM_MAX_SMS_LENGTH;
#else
	    goto not_there;
#endif
	break;
    case 'H':
	break;
    case 'I':
	break;
    case 'J':
	break;
    case 'K':
	break;
    case 'L':
	break;
    case 'M':
	break;
    case 'N':
	break;
    case 'O':
	break;
    case 'P':
	break;
    case 'Q':
	break;
    case 'R':
	break;
    case 'S':
	break;
    case 'T':
	break;
    case 'U':
	break;
    case 'V':
	break;
    case 'W':
	break;
    case 'X':
	break;
    case 'Y':
	break;
    case 'Z':
	break;
    }
    errno = EINVAL;
    return 0;

not_there:
    errno = ENOENT;
    return 0;
}

typedef HV HvObject;
typedef AV AvObject;

MODULE = Gnokii::GSM		PACKAGE = Gnokii::GSM

int
Initialize(ghash)
HvObject *ghash;
PREINIT:
int count=0;
char *model, *device;
CODE:
           RETVAL = GSM_Initialise(SvPV(*hv_fetch(ghash,"model",5,0),na),SvPV(*hv_fetch(ghash,"device",6,0),na),"default",SvIV(*hv_fetch(ghash,"connection",10,0)), NULL);
           while (count++ < 2000 && *GSM_LinkOK == 0)
             usleep(50000);

	   if (*GSM_LinkOK == 0) 
           {
	     fprintf (stderr, _("Hmmm... GSM_LinkOK never went true. Quitting. \n"));
	     exit(-1);
	   }
OUTPUT:
	   RETVAL

int
GetMemoryLocation(ghash,index,location)
HvObject *ghash;
int index;
char *location;
PREINIT:
GSM_Error error;
Perl_GSM_PhonebookEntry *entry;
SV **ssv;
CODE:
           entry = (Perl_GSM_PhonebookEntry *) malloc (sizeof(Perl_GSM_PhonebookEntry));
           ssv = hv_fetch((HV *)SvRV(*hv_fetch(ghash, "MEMORY_TYPES", 12, 0)), location, 2, 0);
           if(ssv != NULL)
           { 
	     memset(entry,0,sizeof(Perl_GSM_PhonebookEntry));
	     entry->MemoryType = SvIV(*ssv);
	     entry->Location = index;
	     error = GSM->GetMemoryLocation(entry);
	     /*	     printf("error=%d location=%s index=%d entry->Name= %s entry->empty=%d\n",error,location,index,entry->Name,entry->Name,entry->Empty);*/
	     RETVAL = error;
	     if(error == GE_NONE)
	     {
	       char l[2];
	       HvObject *hentry;
	       
	       hentry=newHV();
	       l[0] = *location;
	       l[1] = '\0';
	       hentry = newHV();
	       hv_store(hentry, "Name", 4, newSVpv(entry->Name,0), 0);
	       hv_store(hentry, "Number", 6, newSVpv(entry->Number,0), 0);
	       hv_store(hentry, "Memory", 6, newSVpv(l,0), 0);
	       hv_store(hentry, "Group", 5, newSViv(entry->Group), 0);
	       hv_store(ghash, "PBEntry", 7, newRV_inc((SV *)hentry), 0);
	     }
	   }
           else
             RETVAL = GE_INVALIDMEMORYTYPE;
OUTPUT:
	   RETVAL

int
WritePhonebookLocation(ghash, hentry, location)
HvObject *ghash;
HV *hentry;
int location;
PREINIT:
Perl_GSM_PhonebookEntry *entry;
SV **ssv;
CODE:
        RETVAL=GE_NONE;
        entry = (Perl_GSM_PhonebookEntry *)malloc(sizeof(Perl_GSM_PhonebookEntry));
        strcpy(entry->Name, SvPV(*hv_fetch(hentry, "Name", 4, 0),na));
        strcpy(entry->Number, SvPV(*hv_fetch(hentry, "Number", 6, 0),na));

        ssv = hv_fetch((HV *)SvRV(*hv_fetch(ghash, "MEMORY_TYPES", 12, 0)), SvPV(*hv_fetch(hentry, "Memory", 6, 0),na), 2, 0);
        if(ssv != NULL)
        { 
	  entry->MemoryType = SvIV(*ssv);
	  entry->Group = SvIV(*hv_fetch(hentry, "Group",5 ,0));
	  entry->Location=location;
	  RETVAL = GSM->WritePhonebookLocation(entry);
	}
        else
          RETVAL = GE_INVALIDMEMORYTYPE;
OUTPUT:
        RETVAL

int
GetMemStat(ghash, memtype)
HvObject *ghash;
int memtype;
PREINIT:
GSM_MemoryStatus status;
HvObject *stat_fetched;
CODE:
        stat_fetched = newHV();
	status.MemoryType=memtype;
        RETVAL = GSM->GetMemoryStatus(&status);
        if(RETVAL == GE_NONE)
        {
	  hv_store(stat_fetched, "Used", 4, newSViv(status.Used), 0);
	  hv_store(stat_fetched, "Free", 4, newSViv(status.Free), 0);
	}
        hv_store(ghash, "stat_fetched", 12, newRV_inc((SV *)stat_fetched), 0);
OUTPUT:
	RETVAL

int
GetSMSStatus(ghash)
HvObject *ghash;
PREINIT:
GSM_SMSStatus status;
HvObject *smsstat;
CODE:
        RETVAL = GSM->GetSMSStatus(&status);
        if(RETVAL == GE_NONE)
        {
	  smsstat = newHV();
	  hv_store(smsstat, "Unread", 6, newSViv(status.UnRead),0);
	  hv_store(smsstat, "Number", 6, newSViv(status.Number),0);
	  hv_store(ghash, "SMSStatus", 9, newRV_inc((SV *) smsstat), 0);
	}
OUTPUT:
	RETVAL

int
GetSMSCenter(ghash, index)
HvObject *ghash;
int index;
PREINIT:
GSM_MessageCenter *center;
CODE:
        center = (GSM_MessageCenter *)malloc(sizeof(GSM_MessageCenter));
        center->No = index;
        RETVAL = GSM->GetSMSCenter(center);
        if(RETVAL == GE_NONE)
        {
	  HvObject *smsc;

	  smsc = newHV();
	  hv_store(smsc, "No", 2, newSViv(center->No), 0);
	  hv_store(smsc, "Name", 4, newSVpv(center->Name, 0), 0);
	  hv_store(smsc, "Number", 6, newSVpv(center->Number, 0), 0);
	  hv_store(ghash, "SMSCenter", 9, newRV_inc((SV *) smsc), 0);
	}
OUTPUT:
	RETVAL

int
GetSMSMessage(ghash, location, index)
HvObject *ghash;
char *location;
int index;
PREINIT:
GSM_MemoryType memory;
Perl_GSM_SMSMessage *msg;
HvObject *message;
struct tm t;
CODE:
/*        if(!strcasecmp(location,"A"))
	  memory = GMT_SM;
	  else*/
	  memory = GMT_ME;
        msg = (Perl_GSM_SMSMessage *)malloc(sizeof(Perl_GSM_SMSMessage));
        msg->Location = index;
        msg->MemoryType = memory;
	RETVAL = GSM->GetSMSMessage(msg);
	if(RETVAL == GE_NONE)
	{
	  Perl_GSM_DateTime *pgt;

	  pgt = &(msg->Time);
	  message = newHV();
	  GSMDATE_TO_TM(pgt,t,message);
	  hv_store(message,"Length", 6, newSViv(msg->Length),0);
	  hv_store(message,"Validity", 8, newSViv(msg->Validity),0);
	  hv_store(message,"MessageText", 11, newSVpv(msg->MessageText,0),0);
	  hv_store(message,"MessageCenter",13, newSVpv(msg->MessageCenter.Number,0),0);
	  hv_store(message,"Sender", 6, newSVpv(msg->Sender, 0),0);
	  hv_store(message,"Destination", 10, newSVpv(msg->Destination, 0),0);
	  hv_store(message,"Memory", 6, newSVpv(location, 0), 0);
	  hv_store(message,"Type", 4, newSViv(msg->Type),0);
	  hv_store(message,"Status", 6, newSViv(msg->Status),0);
	  hv_store(message,"Class", 5, newSViv(msg->Class),0);
	  hv_store(message,"EightBit", 8, newSViv(msg->EightBit),0);
	  hv_store(message,"Compression", 11, newSViv(msg->Compression),0);
	  hv_store(ghash, "Msg", 3, newRV_inc((SV *)message),0);
	}
        free(msg);
OUTPUT:
	RETVAL

int
DeleteSMSMessage(ghash, memory, location)
HvObject *ghash;
char *memory;
int location;
PREINIT:
Perl_GSM_SMSMessage *message;
GSM_MemoryType mtype;
CODE:
        if(!strcasecmp(memory,"A"))
	  mtype = GMT_SM;
	else
	  mtype = GMT_ME;
       message = (Perl_GSM_SMSMessage *)malloc(sizeof(Perl_GSM_SMSMessage));
       message->Location=location;
       RETVAL = GSM->DeleteSMSMessage(message);
OUTPUT:
       RETVAL

int
SendSMSMessage(ghash, msg)
HvObject *ghash;
HV *msg;
PREINIT:
Perl_GSM_SMSMessage *message;
char *tmp;
HV *rtt;
CODE:
     {  
       message = (Perl_GSM_SMSMessage *)malloc(sizeof(Perl_GSM_SMSMessage));

       if(hv_fetch(msg, "Validity", 8, 0) != NULL)
	 message->Validity = SvIV(*hv_fetch(msg, "Validity", 8, 0));
       else
	 message->Validity = 4320;
       strcpy(message->MessageText, SvPV(*hv_fetch(msg, "MessageText", 11, 0),na));
       if(hv_fetch(msg, "MessageCenter", 13, 0) != NULL)
	 message->MessageCenter.No = SvIV(*hv_fetch(msg, "MessageCentre", 13, 0));
       else
	 message->MessageCenter.No = 1;
       if(hv_fetch(msg, "Class", 5, 0) != NULL)
	 message->Class = SvIV(*hv_fetch(msg, "Class", 5, 0));
       else
	 message->Class = -1;
       if(hv_fetch(msg, "EightBit", 8, 0) != NULL)
	 message->EightBit = 1;
       if(hv_fetch(msg, "RingTone", 8, 0) != NULL)
       {
	 message->UDHType = GSM_RingtoneUDH;
         rtt = hv_fetch(ghash, "RTT", 3, 0);
         memcpy(message->UDH, SvPV(*hv_fetch(rtt, "CONVERTEDRTT", 12, 0), na) + 2, 7);
	 memcpy(message->MessageText, SvPV(*hv_fetch(rtt, "CONVERTEDRTT", 12, 0), na) + 9, SvIV(*hv_fetch(rtt, "SIZE", 4, 0)) - 9);
       }
       else
	 message->UDHType = GSM_NoUDH;
       strcpy(message->Destination , SvPV(*hv_fetch(msg, "Destination", 11, 0), na));
       RETVAL = GSM->SendSMSMessage(message, 0);
    }
OUTPUT:
       RETVAL

int
GetRFLevel(ghash,unit)
HvObject *ghash;
int unit;
PREINIT:
float *f;
CODE:
        f = (float *)malloc(sizeof(float));
        RETVAL = GSM->GetRFLevel(&unit ,f);
        if(RETVAL == GE_NONE)
          hv_store(ghash, "RFLevel", 7, newSVnv((double)*f),0);
OUTPUT:
	RETVAL

int
GetBatteryLevel(ghash,unit)
HvObject *ghash;
int unit;
PREINIT:
float *f;
CODE:
        f = (float *)malloc(sizeof(float));
        RETVAL = GSM->GetBatteryLevel(&unit, f);
        if(RETVAL == GE_NONE)
          hv_store(ghash, "BatteryLevel", 12, newSVnv((double)*f),0);
OUTPUT:
	RETVAL

int
GetPowerSource(ghash)
HvObject *ghash;
PREINIT:
GSM_PowerSource *source;
CODE:
        source = (GSM_PowerSource *)malloc(sizeof(GSM_PowerSource));
        RETVAL = GSM->GetPowerSource(source);
        if(RETVAL == GE_NONE)
        {
          if(*source == GPS_ACDC)
            hv_store(ghash, "PowerSource", 11, newSVpv(strdup("ACDC"),0), 0);
	  else
	    hv_store(ghash, "PowerSource", 11, newSVpv(strdup("Battery"),0), 0);
	}
OUTPUT:
	RETVAL

int 
GetDisplayStatus(ghash)
HvObject *ghash;
PREINIT:
int status;
CODE:
        RETVAL = GSM->GetDisplayStatus(&status);
        if(RETVAL == GE_NONE)
          hv_store(ghash, "DisplayStatus", 13, newSViv(status), 0);
OUTPUT:
	RETVAL


int
EnterSecurityCode(ghash, sechash)
HvObject *ghash;
HV *sechash;
PREINIT:
GSM_SecurityCode *seccode;
CODE:
        seccode = malloc(sizeof(GSM_SecurityCode));
	seccode->Type = SvIV(*hv_fetch(sechash, "Type", 4, 0));
        strcpy(seccode->Code, SvPV(*hv_fetch(sechash, "Code", 4, 0), na));
        RETVAL = GSM->EnterSecurityCode(*seccode);
OUTPUT:
	RETVAL

int
GetSecurityCodeStatus(ghash)
HvObject *ghash;
PREINIT:
int status;
CODE:
        RETVAL = GSM->GetSecurityCodeStatus(&status);
        if(RETVAL == GE_NONE)
          hv_store(ghash, "SecurityCodeStatus", 18, newSViv(status), 0);
OUTPUT:
	RETVAL

int
GetIMEI(ghash)
HvObject *ghash;
PREINIT:
char imei[20];
CODE:
        RETVAL = GSM->GetIMEI(imei);
        if(RETVAL == GE_NONE)
	  hv_store(ghash,"IMEI", 4, newSVpv(imei,0),0);
OUTPUT:
	RETVAL

int 
GetRevision(ghash)
HvObject *ghash;
PREINIT:
char revision[20];
CODE:
        RETVAL = GSM->GetRevision(revision);
        if(RETVAL == GE_NONE)
          hv_store(ghash, "Revision", 8, newSVpv(revision, 0), 0);
OUTPUT:
	RETVAL

int 
GetModel(ghash)
HvObject *ghash;
PREINIT:
char model[8];
CODE:
        RETVAL = GSM->GetModel(model);
        if(RETVAL == GE_NONE)
          hv_store(ghash, "Model", 5, newSVpv(model, 0), 0);
OUTPUT:
	RETVAL

int
GetDateTime(ghash)
HvObject *ghash;
PREINIT:
Perl_GSM_DateTime *date_time;
struct tm t;
CODE:   
        date_time = (Perl_GSM_DateTime *)malloc(sizeof(Perl_GSM_DateTime));
        RETVAL = GSM->GetDateTime(date_time);
        if(RETVAL == GE_NONE)
          GSMDATE_TO_TM(date_time,t,ghash);
OUTPUT:
	RETVAL

int
SetDateTime(ghash,datetime)
HvObject *ghash;
int datetime;
PREINIT:
Perl_GSM_DateTime *dtime;
struct tm *t;
CODE:
        t = (struct tm *)malloc(sizeof(struct tm));
        t = localtime(&datetime);
        dtime = (Perl_GSM_DateTime *)malloc(sizeof(Perl_GSM_DateTime));
        dtime->Year = t->tm_year + 1900;
        dtime->Month = t->tm_mon +1;
        dtime->Day = t->tm_mday;
        dtime->Hour = t->tm_hour;
        dtime->Minute = t->tm_min;
        dtime->Second = t->tm_sec;
        RETVAL = GSM->SetDateTime(dtime);
OUTPUT:
	RETVAL

int
GetAlarm(ghash, alarm_number)
HvObject *ghash;
int alarm_number;
PREINIT:
Perl_GSM_DateTime *date_time;
struct tm t;
HvObject *ahash;
AvObject *aarray;
SV **ssv;
CODE:
        ssv = hv_fetch(ghash, "Alarms", 6, 0);
        if(ssv == NULL)
          aarray=newAV();
	else
        {
	  int j;
	  aarray=(AV *) SvRV(*ssv);
	  j=av_len(aarray);
	  while(j-->1)
	    av_pop(aarray);
	}

        date_time = (Perl_GSM_DateTime *)malloc(sizeof(Perl_GSM_DateTime));
        RETVAL = GSM->GetAlarm(alarm_number,date_time);
        if(RETVAL == GE_NONE)
        {
	  ahash=newHV();
          GSMDATE_TO_TM(date_time,t,ahash);
	  hv_store(ahash, "Enabled", 7, newSViv(date_time->AlarmEnabled),0);
	  av_store(aarray, alarm_number, newRV_inc((SV *)ahash));
	  if(ssv == NULL)
	    hv_store(ghash,"Alarms",6, newRV_inc((SV *)aarray),0);
	}
OUTPUT:
	RETVAL

int
SetAlarm(ghash, index, datetime)
HvObject *ghash;
int index;
int datetime;
PREINIT:
struct tm *t;
Perl_GSM_DateTime *date_time;
CODE:
        t = (struct tm *)malloc(sizeof(struct tm));
        t = localtime(&datetime);
        date_time = (Perl_GSM_DateTime *)malloc(sizeof(Perl_GSM_DateTime));
        date_time->Year = t->tm_year + 1900;
        date_time->Month = t->tm_mon +1;
        date_time->Day = t->tm_mday;
        date_time->Hour = t->tm_hour;
        date_time->Minute = t->tm_min;
        date_time->Second = t->tm_sec;
        date_time->AlarmEnabled = 1;
        RETVAL = GSM->SetAlarm(index,date_time);
OUTPUT:
        RETVAL

int 
DialVoice(ghash, number)
HvObject *ghash;
char *number;
CODE:
        RETVAL=GSM->DialVoice(number);
OUTPUT:
        RETVAL

int
GetIncomingCallNr(ghash)
HvObject *ghash;
PREINIT:
char *number;
CODE:
        RETVAL = GSM->GetIncomingCallNr(number);
        if(RETVAL == GE_NONE)
          hv_store(ghash,"Number",6, newSVpv(number, 0),0);
OUTPUT:
	RETVAL

int
GetNetworkInfo(ghash)
HvObject *ghash;
PREINIT:
GSM_NetworkInfo *ni;
HvObject *ahash;
CODE:
        ni = (GSM_NetworkInfo *)malloc(sizeof(GSM_NetworkInfo));
        RETVAL = GSM->GetNetworkInfo(ni);
        if(RETVAL == GE_NONE)
        {
	  ahash = newHV();
	  hv_store(ahash, "NetworkCode", 11, newSVpv(ni->NetworkCode,0), 0);
	  hv_store(ahash, "CellID", 6, newSVpv(ni->CellID,0), 0);
	  hv_store(ahash, "LAC", 3, newSVpv(ni->LAC,0), 0);
	  hv_store(ghash, "NetworkInfo", 11, newRV_inc((SV *)ahash),0);
	}
OUTPUT:
	RETVAL

int
GetCalendarNote(ghash, index)
HvObject *ghash;
int index;
PREINIT:
GSM_CalendarNote *cn;
HvObject *chash;
struct tm t;
SV **ssv;
CODE:
        cn = (GSM_CalendarNote *)malloc(sizeof(GSM_CalendarNote));
        cn->Location = index;
        RETVAL = GSM->GetCalendarNote(cn);
        if(RETVAL == GE_NONE)
        {
	  Perl_GSM_DateTime *pgt,*agt;

	  pgt = (Perl_GSM_DateTime *)&(cn->Time);
	  agt = (Perl_GSM_DateTime *)&(cn->Alarm);
	  chash = newHV();
	  if(agt != NULL)
	  {
	    GSMDATE_TO_TM(agt ,t, chash);
	    ssv = hv_fetch(chash, "Date", 4, 0);	    
	    hv_store(chash, "Alarm", 5, *ssv, 0);
	  }
	  GSMDATE_TO_TM(pgt, t, chash);
	  hv_store(chash, "Type", 4, newSViv(cn->Type), 0);
	  hv_store(chash, "Text", 4, newSVpv(cn->Text,0), 0);
	  hv_store(chash, "Phone", 5, newSVpv(cn->Phone,0), 0);
 	  hv_store(ghash, "CalendarNote", 12, newRV_inc((SV *)chash), 0);
	}
OUTPUT:
	RETVAL

int
WriteCalendarNote(ghash, cnhash, index)
HvObject *ghash;
HV *cnhash;
int index
PREINIT:
GSM_CalendarNote *cn;
Perl_GSM_DateTime *pgdt;
int datetime;
CODE:
        cn = (GSM_CalendarNote *)malloc(sizeof(GSM_CalendarNote));
        pgdt = (Perl_GSM_DateTime *)malloc(sizeof(Perl_GSM_DateTime));
        cn->Location = index;
        cn->Type = SvIV(*hv_fetch(cnhash, "Type", 4, 0));
        cn->Alarm.Year=1999;
        cn->Alarm.Month=12;
        cn->Alarm.Day=31;
        cn->Alarm.Hour=23;
        cn->Alarm.Minute=58;
        datetime = SvIV(*hv_fetch(cnhash, "Date", 4, 0));
        HASH_TO_GSMDT(pgdt, &datetime);
        memcpy(&(cn->Time),pgdt, sizeof(Perl_GSM_DateTime));
        strncpy(cn->Text, SvPV(*hv_fetch(cnhash, "Text", 4, 0),na ), 20);
        strncpy(cn->Phone, SvPV(*hv_fetch(cnhash, "Phone", 5, 0),na ), 20);
        RETVAL = GSM->WriteCalendarNote(cn);
OUTPUT:
	RETVAL

int
DeleteCalendarNote(ghash, index)
HvObject *ghash;
int index;
PREINIT:
GSM_CalendarNote *cn;
CODE: 
        cn = (GSM_CalendarNote *)malloc(sizeof(GSM_CalendarNote));
        cn->Location = index;
        RETVAL = GSM->DeleteCalendarNote(cn);
OUTPUT:
        RETVAL


int
NetMonitor(ghash, Mode)
HvObject *ghash;
char *Mode;
PREINIT:
unsigned char mode;
char Screen[50];
CODE:
{
/* stolen from gnokii.c \:) */
        if (!strcmp(Mode,"reset"))
	  mode=0xf0;
	else if (!strcmp(Mode,"off"))
	  mode=0xf1;
	else if (!strcmp(Mode,"field"))
	  mode=0xf2;
	else if (!strcmp(Mode,"devel"))
	  mode=0xf3;
	else if (!strcmp(Mode,"next"))
	  mode=0x00;
	RETVAL = GSM->NetMonitor(mode, Screen);
	if(RETVAL == GE_NONE)
	  hv_store(ghash, "Screen", 6, newSVpv(Screen, 0), 0);
}
OUTPUT:
        RETVAL

int
SendDTMF(ghash, String)
HvObject *ghash;
char *String;
CODE:
        RETVAL = GSM->SendDTMF(String);
OUTPUT:
	RETVAL

int
GetBitmap(ghash, type, callgroup)
HvObject *ghash;
char *type;
char *callgroup
PREINIT:
HV *bitmaphash;
GSM_Bitmap *bptr;
CODE:
        bptr = malloc(sizeof(GSM_Bitmap));
        bptr->type=GSM_None;
/* stolen from gnokii.c */
        if (strcmp(type,"op")==0)
          bptr->type=GSM_OperatorLogo;
        if (strcmp(type,"caller")==0) 
        {
	    bptr->number=callgroup[0]-'0';
	    bptr->type=GSM_CallerLogo;
	    if ((bptr->type<0)||(bptr->type>9)) 
		bptr->type=0;
	}
        if (strcmp(type,"startup")==0)
          bptr->type=GSM_StartupLogo;
  
	if (bptr->type!=GSM_None) 
        {
	    RETVAL = GSM->GetBitmap(bptr);
	    if(RETVAL == GE_NONE)
	    {
		bitmaphash = newHV();
		hv_store(bitmaphash, "height", 6, newSViv(bptr->height) ,0);
		hv_store(bitmaphash, "width", 5, newSViv(bptr->width) ,0);
		hv_store(bitmaphash, "size", 4, newSViv(strlen(bptr->bitmap)) ,0);
		hv_store(bitmaphash, "netcode", 7, newSVpv(bptr->netcode, 0), 0);
		hv_store(bitmaphash, "text", 4, newSVpv(bptr->text, 0), 0);
		hv_store(bitmaphash, "bitmap", 6, newSVpv(bptr->bitmap, 0), 0);
		hv_store(bitmaphash, "number", 6, newSVpv(&(bptr->number), 0), 0);
		hv_store(bitmaphash, "ringtone", 8, newSVpv(&(bptr->ringtone), 0), 0);
		hv_store(ghash, "Bitmap", 6, newRV_inc((SV *) bitmaphash), 0);
	    }
	}
OUTPUT:
	RETVAL

int 
SetBitmap(ghash)
HvObject *ghash;
PREINIT:
HV *bitmaphash;
GSM_Bitmap *bptr;
CODE:
        bitmaphash = (HV *) SvRV(*hv_fetch(ghash, "Bitmap", 6, 0));
        bptr = malloc(sizeof(GSM_Bitmap));
        bptr->height = SvIV(*hv_fetch(bitmaphash,"height", 6, 0));
        bptr->width = SvIV(*hv_fetch(bitmaphash,"width", 5, 0));
        bptr->type = SvIV(*hv_fetch(bitmaphash,"type", 4, 0));
        strcpy(bptr->netcode, SvPV(*hv_fetch(bitmaphash,"netcode", 7, 0), na));
	strcpy(bptr->text, SvPV(*hv_fetch(bitmaphash, "text", 4, 0), na));
        strcpy(bptr->bitmap, SvPV(*hv_fetch(bitmaphash, "bitmap", 6, 0), na));
        bptr->number = *(SvPV(*hv_fetch(bitmaphash, "number", 6, 0), na));
        bptr->ringtone = *(SvPV(*hv_fetch(bitmaphash, "ringtone", 8, 0), na));
        bptr->size = strlen(bptr->bitmap);
        RETVAL = GSM->SetBitmap(bptr);
OUTPUT:
        RETVAL

int
Reset(type)
char *type;
PREINIT:
unsigned char real_type;
CODE:
        RETVAL = 0;
        if(!strcmp(type, "soft"))
	  real_type = 0x03;
	else
	{
	  if(!strcmp(type, "hard"))
	    real_type = 0x04;
	  else
	    RETVAL = -1;
	}
	if(RETVAL == 0)
	  RETVAL = GSM->Reset(real_type);
OUTPUT:
        RETVAL

int
GetProfile(ghash, number)
HvObject *ghash;
int number;
PREINIT:
GSM_Profile *Profile;
HV *profhash;
CODE:
        Profile = malloc(sizeof(GSM_Profile));
	Profile->Number = number;
	RETVAL = GSM->GetProfile(Profile);
	if(RETVAL == GE_NONE)
	{
	  profhash = newHV();
	  hv_store(profhash, "Name", 4, newSVpv(Profile->Name, 0), 0);
	  hv_store(profhash, "Number", 10, newSViv(Profile->Number), 0);
	  hv_store(profhash, "KeypadTone", 10, newSViv(Profile->KeypadTone), 0);
	  hv_store(profhash, "Lights", 6, newSViv(Profile->Lights), 0);
	  hv_store(profhash, "CallAlert", 9, newSViv(Profile->CallAlert), 0);
	  hv_store(profhash, "Ringtone", 8, newSViv(Profile->Ringtone), 0);
	  hv_store(profhash, "Volume", 6, newSViv(Profile->Volume), 0);
	  hv_store(profhash, "MessageTone", 11, newSViv(Profile->MessageTone), 0);
	  hv_store(profhash, "WarningTone", 11, newSViv(Profile->WarningTone), 0);
	  hv_store(profhash, "Vibration", 9, newSViv(Profile->Vibration), 0);
	  hv_store(profhash, "CallerGroups", 12, newSViv(Profile->CallerGroups), 0);
	  hv_store(profhash, "AutomaticAnswer", 15, newSViv(Profile->AutomaticAnswer), 0);
	  hv_store(ghash, "Profile", 7, newRV_inc((SV *)profhash), 0);
	}
OUTPUT:
	RETVAL

int
SetProfile(ghash)
HvObject *ghash;
PREINIT:
GSM_Profile *Profile;
HV *profhash;
CODE:
        profhash = (HV *)SvRV(*hv_fetch(ghash, "Profile", 7, 0));
	Profile = malloc(sizeof(GSM_Profile));
	Profile->Number = SvIV(*hv_fetch(profhash, "Number", 6, 0));
	strcpy(Profile->Name, SvPV(*hv_fetch(profhash, "Name", 4, 0), na));
	Profile->KeypadTone = SvIV(*hv_fetch(profhash, "KeypadTone", 10, 0));
	Profile->Lights = SvIV(*hv_fetch(profhash, "Lights", 6, 0));
	Profile->CallAlert = SvIV(*hv_fetch(profhash, "CallAlert", 9, 0));
	Profile->Ringtone = SvIV(*hv_fetch(profhash, "Ringtone", 8, 0));
	Profile->Volume = SvIV(*hv_fetch(profhash, "Volume", 6, 0));
	Profile->MessageTone = SvIV(*hv_fetch(profhash, "MessageTone", 11, 0));
	Profile->WarningTone = SvIV(*hv_fetch(profhash, "WarningTone", 11, 0));
	Profile->Vibration = SvIV(*hv_fetch(profhash, "Vibration", 9, 0));
	Profile->CallerGroups = SvIV(*hv_fetch(profhash, "CallerGroups", 12, 0));
	Profile->AutomaticAnswer = SvIV(*hv_fetch(profhash, "AutomaticAnswer", 15, 0));
	RETVAL = GSM->SetProfile(Profile);
OUTPUT:
        RETVAL

int
SendRingToneFile(ghash, filename, number)
HvObject *ghash;
char *filename;
char *number
PREINIT:
GSM_Ringtone *ringtone;
CODE:
        ringtone = (GSM_Ringtone *)malloc(sizeof(GSM_Ringtone));
        RETVAL = GSM->SendRingtone(ringtone, number);
OUTPUT:
        RETVAL

void
createLogoSMS(ghash,msg,filename,type)
HvObject *ghash;
HvObject *msg;
char *filename;
char *type;
PREINIT:
char Data[7] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
char UserDataHeader[7] = {0x06, 0x05, 0x04, 0x15, 0x00, 0x00, 0x00};
int current;
char *message;
GSM_Bitmap *bitmap;
CODE:
       current = 0;
       bitmap = (GSM_Bitmap *)malloc(sizeof(GSM_Bitmap));
       hv_store(msg, "Class", 5, newSViv(1), 0);
       hv_store(msg, "Validity", 8, newSViv(4320), 0);
       hv_store(msg, "MessageCenter", 13, newSViv(1), 0);
       hv_store(msg, "EightBit", 8, newSViv(1), 0);
       hv_store(msg, "Logo", 4, newSVpv(type, 0), 0);
       GSM_ReadBitmapFile(filename, bitmap);
       if(!strcmp(type, "op"))
       {
         UserDataHeader[4] = 0x82;
	 Data[current++] = ((bitmap->netcode[1] & 0x0f) << 4) | (bitmap->netcode[0] & 0xf);
	 Data[current++] = 0xf0 | (bitmap->netcode[2] & 0x0f);
	 Data[current++] = ((bitmap->netcode[5] & 0x0f) << 4) | (bitmap->netcode[4] & 0xf);
       }
       else if(!strcmp(type, "caller"))
         UserDataHeader[4] = 0x83;
       current++;
       Data[current++] = bitmap->width;
       Data[current++] = bitmap->height;
       Data[current++] = 0x01;
       message = (char *) malloc(sizeof(char) * current);
       memcpy(message, Data, current);
       memcpy(message + current, bitmap->bitmap, bitmap->size);
       hv_store(msg, "MessageText", 11, newSVpv(message, 0), 0);

void
finish()
     CODE:
           GSM->Terminate();

double
constant(name,arg)
	char *		name
	int		arg

